# Icecast video streaming with OBS

Test configuration steps for streaming video from an Icecast server with OBS as the source.

The intention is to be able to self-host a video stream that could be used for a variety of purposes, such as "watch/view together" use.

The configuration below has been tested to keep each connected viewer closely synced to the same point in the stream. The difference in sync between viewers stays roughly between 0-2 seconds in testing. If this isn't of importance to you, then you may benefit from increasing the Icecast `queue-size` and `burst-size`.

A self-signed SSL certificate will be generated in this example, with SSL/TLS used for the `OBS > Icecast` connection, and for the `Viewer > Icecast` connections. The Icecast server will be streaming publically unless you setup firewalling to do otherwise, or if you password protect the viewer connections (by putting Icecast behind a password protected NGINX proxy or similar).

### Troubleshooting

* The media player disconnects mid stream or struggles to start the stream, and you don't think it's a bandwidth issue.
    - Try increasing the Icecast `queue-size` and `burst-size`, or reducing the `Video Bitrate` within OBS streaming to Icecast.

* As soon as you click `Start Recording` in OBS, it errors out with `Error number -10054 occurred`.

    - In testing, this occured when trying to connect to Icecast over SSL/TLS, but only on Windows 10 and not under Linux (Fedora 34). OBS 27.0.1 was used on both systems. This might be due to potential differences between FFmpeg versions that end up being used by OBS across these two tested OS setups, this is just a guess though.

        - Disable SSL/TLS under Icecast (just on the icecast `listener` you connect OBS to, you can still have another `listener` setup in the Icecast configuration for any viewers to connect with SSL/TLS if desired. You can use the adjusted config below to connect OBS to Icecast on port 8000 without SSL/TLS.

               # Remove the <ssl> tag from the <listen-socket> section, and set the <port> to 8000

               <listen-socket>
                   <port>8000</port>
                   <bind-address>ip-address-goes-here</bind-address>
               </listen-socket>

               # Additionally, remove <ssl-certificate> from the <paths> section

        - Remove the `tls=1` option from the the OBS `Muxer Settings`, or set try setting it to `tls=0`:

               File > Settings > Output

               Output Mode                    = Advanced
               Select the "Recording" tab
                   Muxer Settings             = content_type=video/m2ts

* How can I get this to use a browser friendly video container so I can stream within a web browser?
    - You can try using `webm` for the `Container Format` inside OBS, with `libvpx` or `libvpx-vp9` as the `Video Encoder`. I encountered performance issues on the encoding end when testing this, but your milage may vary.

## Setup Icecast

This was tested on a Vultr 1 vCPU / 2GB RAM server running Fedora 33 x64.

1. Disable firewalld, as the cloud provider's firewall around the instance will suffice for testing. Just make sure port 22 & 443 are accessible.
    - ```$ systemctl stop firewalld```
1. Install Icecast
    - `$ dnf install -y icecast`
1. Generate a self-signed SSL certificate.
    - `$ openssl req -x509 -nodes -days 365 -subj '/CN=icecastserver/O=icecastserver/C=SG' -newkey rsa:4096 -keyout /usr/share/icecast/ssl.crt -out /usr/share/icecast/ssl.crt`
    - `$ chmod 644 /usr/share/icecast/ssl.crt`
1. Make sure the following items are set within the Icecast configuration, within their respective sections.
    - `$ vim /etc/icecast/icecast.xml`
    ```
        <limits>
    	<queue-size>2046000</queue-size>
    	<burst-size>1024000</burst-size>
    </limits>

    <listen-socket>
	    <port>443</port>
        <bind-address>ip-address-goes-here</bind-address>
        <ssl>1</ssl>
    </listen-socket>

    <paths>
	    <ssl-certificate>/usr/share/icecast/ssl.crt</ssl-certificate>
    </paths>

    <authentication>
	    <source-password>password-goes-here</source-password>
	    <relay-password>password-goes-here</relay-password>
	    <admin-user>admin</admin-user>
	    <admin-password>password-goes-here</admin-password>
    </authentication>
    ```
1. Start Icecast
    -   `$ icecast -b -c /etc/icecast.xml`

## Setup OBS

1. Setup OBS to stream to Icecast as follows:

    ```
    File > Settings > Output
        Output Mode          = Advanced

        Select the "Recording" tab
            Type                       = Custom Output (FFmpeg)
            File path or URL           = icecast://<username>:<password>@<ip-address>:<port>/<mountpoint>.ts
            Container Format           = mpegts
            Muxer Settings             = content_type=video/m2ts tls=1
            Video Bitrate              = 6000 Kbps
            Keyframe interval (frames) = 250
            Show all codecs            = Ticked
            Video Encoder              = libx264
            Audio Bitrate              = 192 Kbps
            Audio Encoder              = opus (libopus)
    ```
1. Click `Start Recording` to start streaming from OBS to the Icecast server.

## Connect to the Icecast stream

1. Open the `.ts` URL of the Icecast server using your media player of choice (MPV, VLC, PotPlayer etc)
    - `mpv https://<ip-address>/<mountpoint>.ts`

